@@ -84,7 +84,7 @@ Assert.prototype.getVersion = function () {
 };
 
 Assert.prototype.renderArgs = function () {
-  return String(this.conditional);
+  if (this.conditional) return String(this.conditional);
 };
 
 Assert.prototype.upgrade = function (confVer, curVer, conf) {
