@@ -15,6 +15,7 @@ Components.utils.import('resource://indexdata/util/jsonPathHelper.js');
 Components.utils.import('resource://indexdata/util/htmlHelper.js');
 Components.utils.import('resource://indexdata/util/logging.js');
 Components.utils.import('resource://indexdata/thirdparty/jsonPath.js');
+Components.utils.import("resource://indexdata/runtime/core.js");
 
 
 var logger = logging.getLogger();
@@ -164,7 +165,7 @@ Httpclient.prototype.run = function (task) {
   }
   var hdr = this.getHeaders(task);
   // Inside the engine, we can just fetch the page.
-  if ( this.task.connector.insideEngine() ) {
+  if (!core.inBuilder) {
     var dom = xmlHelper.fetchDoc( url, false, false, reqtype, req, hdr );
     this.task.connector.setPageOverride(dom,url);
     return;
