@@ -358,6 +358,16 @@ Transform.prototype.upgrade = function (confVer, curVer, conf) {
 
 // Capability flags
 Transform.prototype.capabilityFlagDefault = function ( flag ) {
+    if (this.task.name === "init" && flag.substring(0, 5) == "init-") {
+	var usedArgs = this.conf.usedArgs;
+	//this.task.debug("init flag '" + flag + "' within init task: compare with " + usedArgs.length + " used args: " + JSON.stringify(usedArgs));
+	for (var i = 0; i < usedArgs.length; i++) {
+	    if (usedArgs[i] === flag.substring(5)) {
+		return true;
+	    }
+	}
+    }
+
   var result = jsonPathHelper.capabilityFlagDefault(this.conf.in, flag,
     "$.input", "index");
   for (let i = 0; i < this.conf.outputs.length; i++) {
