@@ -1,5 +1,6 @@
 var EXPORTED_SYMBOLS = ["ConnectorFramework"];
 
+Components.utils.import('resource://indexdata/runtime/core.js');
 Components.utils.import('resource://indexdata/ui/app.js');
 Components.utils.import('resource://indexdata/ui/repoClient.js');
 Components.utils.import('resource://indexdata/ui/taskPane.js');
@@ -9,6 +10,7 @@ Components.utils.import('resource://indexdata/ui/UrlManipulator.js');
 
 var ConnectorFramework = {
   app: app,
+  core: core,
   repoClient: repoClient,
   taskPane: taskPane,
   filterButton: filterButton,
