@@ -494,7 +494,7 @@ var xmlHelper = {
         var els = node.getElementsByTagName(tag);
         var elsLen = els.length;
         var pattern = new RegExp( "\\b" + searchClass + "\\b" );
-        for (i = 0, j = 0; i < elsLen; i++) {
+        for (let i = 0, j = 0; i < elsLen; i++) {
             if ( pattern.test(els[i].className) ) {
                 classElements[j] = els[i];
                 j++;
@@ -689,7 +689,22 @@ var xmlHelper = {
       return xmlStr;
     },
 
-    
+    // Return a string that contains a node name and bit of the text content
+    // suitably cleaned for debug logs. 
+    nodeToString: function(node, maxlen){
+      if (!maxlen)
+        maxlen = 20;
+      if (!node)
+        return "(null)";
+      var s = node.localName || node.nodeName || "(??)";
+      if (node.textContent) {
+        var c = " " + node.textContent;
+        c = c.replace( /\s+/g, " ");
+        s += c.substr(0,maxlen);
+      }
+      return s;
+    },
+
     // The resolveXIncludes() function was originally lifted from
     //  https://developer.mozilla.org/en/XInclude
     // but has needed some modifications to work in the CF's FireFox environment.
@@ -727,7 +742,7 @@ var xmlHelper = {
                     // It's not an absolute HREF
                     var old = href;
                     href = baseDir + "/" + href;
-                    logger.log(Level.INFO, "upgraded relative href '" + old + "' to " + href);
+                    //logger.log(Level.INFO, "upgraded relative href '" + old + "' to " + href);
                 }
                 var parse = xinclude.getAttribute('parse');
                 var xpointer = xinclude.getAttribute('xpointer');
@@ -821,7 +836,7 @@ var xmlHelper = {
                                     responseNodes = [response.documentElement]; // Put in array so can be treated the same way as the above
                                 }
                                 // PREPEND ANY NODE(S) (AS XML) -- REMOVE XINCLUDE LATER
-                                for (j=0; j < responseNodes.length ; j++) {
+                                for (let j=0; j < responseNodes.length ; j++) {
                                     xincludeParent.insertBefore(responseNodes[j], xinclude);
                                 }
                              }
@@ -856,14 +871,14 @@ var xmlHelper = {
                     if (!xiFallback) throw new Error("XInclude can't include '" + href + "': " + e);
                     var xiFallbackChildren = xiFallback.childNodes;
                     // PREPEND ANY NODE(S) -- REMOVE XINCLUDE LATER
-                    for (j=0; j < xiFallbackChildren.length ; j++) {
+                    for (let j = 0; j < xiFallbackChildren.length ; j++) {
                         xincludeParent.insertBefore(xiFallbackChildren[j], xinclude);
                     }
                 }
                 logger.log(Level.DEBUG, "finished XInclude node #" + (i+1) + " of " + xincludes.length);
             }
             // Delete in reverse order so shrinking .length does not prematurely end the loop
-            for (i = xincludes.length-1; i >= 0; i--) {
+            for (let i = xincludes.length-1; i >= 0; i--) {
                 logger.log(Level.DEBUG, "deleting XInclude node #" + (i+1) + " of " + xincludes.length);
                 xincludes[i].parentNode.removeChild(xincludes[i]);
             }
