@@ -439,7 +439,7 @@ Xpattern.prototype.dumpString = function ( indent ) {
   if (indent>=0) {
     nextindent++;
     newline="\n";
-    for(i=0;i<indent;i++) {
+    for(let i=0;i<indent;i++) {
       indentation = indentation + "  ";
     }
   }
