@@ -35,7 +35,7 @@ Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/logging.js');
 var logger = logging.getLogger();
 var debug = false;  // Produces a very detailed dump of the process
-//debug = true;
+debug = true;
 
 
 // constructor. Takes a Dom node that indicates the area to start from
@@ -51,7 +51,7 @@ function XpatternMaker( startnode ) {
 // Add a node to be included in the pattern
 XpatternMaker.prototype.addNode =
     function (node, variable, cardinality, attrs, group, modifiers ) {
-      targ = { "node": node,
+      var targ = { "node": node,
               "variable": variable,
               "cardinality": cardinality,
               "attr" : attrs,
@@ -251,7 +251,7 @@ XpatternMaker.prototype._findmatch = function (n) {
             return sp;
         }
     } else { // node didn't match
-        cp = this._findmatch(n.firstChild);
+        var cp = this._findmatch(n.firstChild);
         if (cp != null)  { // but its children did
             return cp;
         }
@@ -436,6 +436,13 @@ XpatternMaker.prototype._makegroup = function(xp, parent, prev, groupno) {
     logger.debug("xpmk_mk: Check for " + group.dumpString(-1) +
       " returns " + ok );
   // convert into a OR-bag if that's what we need
+    // TODO - Check if we can skip some nodes, for example
+    //  ( U $x | U )+
+    // The second U above can be removed, because
+    //  - or-bags repeat
+    //  - the first U has no conditions (siblings, attrs, patterns)
+    // Actually, the deduplicating logic below is a good start,
+    // probably it can be refined.
   var seen = {}; // deduplicate
   if ( ok && this.groups[groupno].grouptype == "|" ) { 
     var ornode = new Xpattern();  // the or-node inside the group
@@ -498,7 +505,13 @@ XpatternMaker.prototype.unitTest = function ( ) {
           "<TD>Description2</TD>" +
         "</TR>"+
       "</TABLE>"+
-      "</div>"+
+    "</div>"+
+    "<div>" + // div 3 - test for CP-3412, optimized or-bags
+      "<H4>Title</H4>" +
+      "<U>First Author </U>" +
+      "<U>Second Author</U>" + 
+      "<B>Publisher</B>" +
+    "</div>"+
     "</body>";
 
   const tests = [
@@ -745,15 +758,26 @@ XpatternMaker.prototype.unitTest = function ( ) {
       ],
       ex: "( TR  { TD $author }  | TR  { TD $publisher }  | TR  { TD $description } ) +",
     },
+    { name: "Orbag-6",
+      // Test case for CP-3412, should not get confused about the second U
+      targets: [
+        { p:"//body/div[3]/H4", v: "title" },
+        { p:"//body/div[3]", group:1, parent:0,
+                   type:"|", cardinality:"+" },
+        { p:"//body/div[3]/U", group:1, v: "author" },
+        { p:"//body/div[3]/B", group:1, v: "publisher" },
+      ],
+      ex: "H4 $title : ( U $author  | B $publisher ) +",
+      //ex: "H4 $title : ( U $author | U | B $publisher ) +",
+      fail: "Should not add the plain U tag. CP-3412 ",
+    },
 /*    
-      targets: [ { p:"//body/div[1]/A", v: "title" },
-                 { p:"//body/div[1]/B", v: "publisher" }  ],
-
-      "<A href='http://indexdata.com'>Title</A><br/>"+
-      "<U>Author</U><br/>" +
-      "<B>Publisher</B><br/>" +
-      "<I>Description</I><br/>" +
-
+    "<div>" + // div 3 - test for CP-3412, optimized or-bags
+      "<h4>Title</h4>" +
+      "<U>First Author </U>" +
+      "<U>Second Author</U>" +
+      "<B>Publisher</B>" +
+    "</div>"+
 */    
   ]; // test array
 
@@ -761,7 +785,7 @@ XpatternMaker.prototype.unitTest = function ( ) {
   function cleanstr(s) {
     s = s.replace( /^\s*/,"" );  // leading
     s = s.replace( /\s*$/,"" );  // trailing
-    s = s.replace( /\s+/, " ");  // long spaces
+    s = s.replace( /\s+/g, " ");  // long spaces
     return s;
   };
   
