@@ -1,15 +1,23 @@
 #!/bin/sh
-# assumes pwd is engine/src
+# assumes pwd is engine/src, and that MP has been started.
+
 
 DIR=`dirname $0`            # ../../connectors/cf-zserver-regression
 PLAINDIR=`basename $DIR`    # cf-zserver-regression
 
-echo "running $0"
 GRET=0
 : ${diff_opt="-u"}
 : ${REGRESSION_FATAL=false}
 
-CONNECTORLIST=`cd $DIR/..; ls *.cf $PLAINDIR/*.cf`
+if [ $1 ]
+then
+  echo "running $0, only for $1"
+  CONNECTORLIST=$1
+else
+  echo "running $0"
+  CONNECTORLIST=`cd $DIR/..; ls *.cf $PLAINDIR/*.cf`
+fi
+
 for C in $CONNECTORLIST
 do
     CNAME=`basename $C .cf`
@@ -32,11 +40,18 @@ do
     while read QRY1
     do  # for each line in the file
       echo "======================" >> $LOG
-      QRY=`echo $QRY1 | sed 's/^#.*$//' `  # strip comment lines
+      # If line starts with '?', take first word to be dbname params
+      # and the rest to be the query. Pass the params with the connector name.
       echo $QRY1  >> $LOG
+      QRY2=`echo $QRY1 | sed 's/^#.*$//' `  # strip comment lines
+      QRY=`echo $QRY2 | sed 's/^\?[^ ]* *//'`  # remove the params
+      DBPARM=`echo $QRY2 | sed -r 's/(\?[^ ]*)?( ?.*)/\1/'` # keep just the db
+      #echo "DB='$DBPARM'" >> $LOG   # ###
+      #echo "Q='$QRY'" >> $LOG       # ###
+      #echo "C-DB='$CONNECTOR$DBPARM'" >> $LOG # ###
       if [ ! -z "$QRY" ]
       then 
-        zoomsh "connect localhost:9070" "set databaseName $CONNECTOR" \
+        zoomsh "connect localhost:9070" "set databaseName $CONNECTOR$DBPARM" \
                   "set count 2" "search $QRY" "quit" >>$LOG
         test $? -ne 0 && RET=1
       fi
