@@ -6,6 +6,7 @@
  * \file cf_engine.cpp
  * \brief Implementation of CF_Embed / CF_Engine.
  */
+#include "mozilla-config.h"
 #include "cf_config.h"
 #include <stdlib.h>
 #include <errno.h>
@@ -176,7 +177,7 @@ void CF_Embed::main()
 
     mozApp = new MozApp(logger);
 
-    if (mozApp->init(app_path, profile_path))
+    if (NS_FAILED(mozApp->init(app_path, profile_path)))
     {
         write_response("FAIL\n", 5);
         // we not listening to pipe input by exiting here
@@ -313,7 +314,6 @@ void CF_Embed::write_response(const void *buf, size_t count)
 
 void CF_Embed::input_handler(GIOChannel *source)
 {
-    char cmd[20], taskName[160];
     gchar *str_return;
     gsize length;
     GError *gerror = 0;
@@ -338,116 +338,128 @@ void CF_Embed::input_handler(GIOChannel *source)
     }
     if (length)
     {
-	int off = 0;
-        int r = sscanf(str_return, "%19s %159s%n", cmd, taskName, &off);
 	WRBUF w = wrbuf_alloc(); // zero-length means no response (yet)
+        char *cp = str_return;
+        const char *cmd = cp;
+        const char *first_arg = "";
+        const char *extra_args = "";
 
-        if (r < 1)
-            wrbuf_printf(w, "FAIL -\n");
-	else
-	{
-	    if (str_return[length-1] == '\n')
-		str_return[length-1] = '\0';
-	    const char *args = str_return + off;
-	    while (*args == ' ')
-		args++;
-	    if (!strcmp(cmd, "load_cf"))
+        if (str_return[length-1] == '\n')
+            str_return[length-1] = '\0';
+
+        // split into 3 tokens, cmd, first_arg, extra_args - separted by blanks
+        // Perform in-place null'ing..
+        for (; *cp && *cp != ' '; cp++)
+            ;
+        if (*cp)
+        {
+            *cp++ = '\0'; // terminate cmd
+            for (first_arg = cp; *cp && *cp != ' '; cp++)
+                ;
+            if (*cp)
             {
-                char *errormessage = 0;
-		if (connector->loadConf(taskName, args, &errormessage))
-                    wrbuf_printf(w, "OK\n");
-                else
-                {
-                    wrbuf_printf(w, "FAIL load_cf");
-                    if (errormessage)
-                        wrbuf_printf(w, ": %s", errormessage);
-                    wrbuf_puts(w, "\n");
-                }
+                *cp++ = '\0'; // terminate first_arg
+                while (*cp == ' ')
+                    cp++;
+                extra_args = cp;
             }
-            else if (!strcmp(cmd, "save_cf"))
-            {
-                char *errormessage = 0;
-		if (connector->saveConf(taskName, &errormessage))
-                    wrbuf_printf(w, "OK\n");
-                else
-                {
-                    wrbuf_printf(w, "FAIL save_cf");
-                    if (errormessage)
-                        wrbuf_printf(w, ": %s", errormessage);
-                    wrbuf_puts(w, "\n");
-                }
-            }
-            else if (!strcmp(cmd, "run_script"))
-            {
-                char *errormessage = 0;
-		if (connector->runScript(taskName, &errormessage))
-                    wrbuf_printf(w, "OK\n");
-                else
-                {
-                    wrbuf_printf(w, "FAIL run_script");
-                    if (errormessage)
-                        wrbuf_printf(w, ": %s", errormessage);
-                    wrbuf_puts(w, "\n");
-                }
-            }
-	    else if (!strcmp(cmd, "run_task"))
-	    {
-                yaz_timing_start(task_timing);
-                wrbuf_rewind(current_task);
-                wrbuf_puts(current_task, taskName);
-		connector->runTask(taskName, false,
-                                   args, 0, task_result_handler);
-	    }
-	    else if (!strcmp(cmd, "run_task_opt"))
-	    {
-                yaz_timing_start(task_timing);
-                wrbuf_rewind(current_task);
-                wrbuf_puts(current_task, taskName);
-		connector->runTask(taskName, true,
-                                   args, 0, task_result_handler);
-	    }
-	    else if (!strcmp(cmd, "run_tests"))
-	    {
-                yaz_timing_start(task_timing);
-                wrbuf_rewind(current_task);
-                wrbuf_puts(current_task, taskName);
-		connector->runTests(taskName, 0, test_task_done,
-                                    test_finished);
-            }
-            else if (!strcmp(cmd, "unit_test"))
-            {
-                yaz_timing_start(task_timing);
-                wrbuf_rewind(current_task);
-                wrbuf_puts(current_task, args);
-		if (connector->unitTest(args))
-                    wrbuf_printf(w, "OK\n");
-                else
-                    wrbuf_printf(w, "FAIL\n");
-            }
-            else if (!strcmp(cmd, "dom_string"))
-            {
-                logger->printf("engine", YLOG_DEBUG,
-                               "dom_string command handler.\n");
-                char *dom_str = 0;
-		if (connector->DOMString(&dom_str))
-                {
-                    get_CF_Embed()->result_handler(true, dom_str);
-                }
-                else
-                    wrbuf_printf(w, "FAIL\n");
-            }
-            else if (!strcmp(cmd, "quit"))
-            {
+        }
+        if (!strcmp(cmd, "load_cf"))
+        {
+            char *errormessage = 0;
+            if (connector->loadConf(first_arg, extra_args, &errormessage))
                 wrbuf_printf(w, "OK\n");
-                delete mozApp;
-                mozApp = 0;
-                gtk_main_quit();
-            }
             else
             {
-                wrbuf_printf(w, "FAIL %s\n", cmd);
+                wrbuf_printf(w, "FAIL load_cf");
+                if (errormessage)
+                    wrbuf_printf(w, ": %s", errormessage);
+                wrbuf_puts(w, "\n");
             }
-	}
+        }
+        else if (!strcmp(cmd, "save_cf"))
+        {
+            char *errormessage = 0;
+            if (connector->saveConf(first_arg, &errormessage))
+                wrbuf_printf(w, "OK\n");
+            else
+            {
+                wrbuf_printf(w, "FAIL save_cf");
+                if (errormessage)
+                    wrbuf_printf(w, ": %s", errormessage);
+                wrbuf_puts(w, "\n");
+            }
+        }
+        else if (!strcmp(cmd, "run_script"))
+        {
+            char *errormessage = 0;
+            if (connector->runScript(first_arg, &errormessage))
+                wrbuf_printf(w, "OK\n");
+            else
+            {
+                wrbuf_printf(w, "FAIL run_script");
+                if (errormessage)
+                    wrbuf_printf(w, ": %s", errormessage);
+                wrbuf_puts(w, "\n");
+            }
+        }
+        else if (!strcmp(cmd, "run_task"))
+        {
+            yaz_timing_start(task_timing);
+            wrbuf_rewind(current_task);
+            wrbuf_puts(current_task, first_arg);
+            connector->runTask(first_arg, false,
+                               extra_args, 0, task_result_handler);
+        }
+        else if (!strcmp(cmd, "run_task_opt"))
+        {
+            yaz_timing_start(task_timing);
+            wrbuf_rewind(current_task);
+            wrbuf_puts(current_task, first_arg);
+            connector->runTask(first_arg, true,
+                               extra_args, 0, task_result_handler);
+        }
+        else if (!strcmp(cmd, "run_tests"))
+        {
+            yaz_timing_start(task_timing);
+            wrbuf_rewind(current_task);
+            wrbuf_puts(current_task, first_arg);
+            connector->runTests(first_arg, 0, test_task_done,
+                                test_finished);
+        }
+        else if (!strcmp(cmd, "unit_test"))
+        {
+            yaz_timing_start(task_timing);
+            wrbuf_rewind(current_task);
+            wrbuf_puts(current_task, extra_args);
+            if (connector->unitTest(extra_args))
+                wrbuf_printf(w, "OK\n");
+            else
+                wrbuf_printf(w, "FAIL\n");
+        }
+        else if (!strcmp(cmd, "dom_string"))
+        {
+            logger->printf("engine", YLOG_DEBUG,
+                           "dom_string command handler.\n");
+            char *dom_str = 0;
+            if (connector->DOMString(&dom_str))
+            {
+                get_CF_Embed()->result_handler(true, dom_str);
+            }
+            else
+                wrbuf_printf(w, "FAIL\n");
+        }
+        else if (!strcmp(cmd, "quit"))
+        {
+            wrbuf_printf(w, "OK\n");
+            delete mozApp;
+            mozApp = 0;
+            gtk_main_quit();
+        }
+        else
+        {
+            wrbuf_printf(w, "FAIL %s\n", cmd);
+        }
         fflush(stdout);
         if (wrbuf_len(w))
             write_response(wrbuf_buf(w), wrbuf_len(w));
