@@ -39,7 +39,7 @@ public:
      */
     CF_Engine *create(enum CF_Engine::Flags, const char *proxy,
                       const char *loglevel,
-                      CF_Logger *logger, int fd);
+                      CF_Logger *logger, int fd, int capt_stdout);
 
     WRBUF get_cf_file(WRBUF username, WRBUF password,
                       const char *db, int *http_status);
