@@ -1,5 +1,5 @@
 /* This file is part of the Connector Framework
- * Copyright (C) 2008-2012 Index Data
+ * Copyright (C) 2008-2013 Index Data
  * See the file LICENSE for details.
  */
 
@@ -49,7 +49,7 @@ int main(int argc, char **argv)
     }
     wrbuf_rewind(w);
     if (xml_parse)
-        make_xml_al(n, w, 0);
+        json_to_xml(n, w, "opt");
     else
         json_write_wrbuf(n, w);
     json_remove_node(n);
