@@ -51,9 +51,11 @@ namespace metaproxy_1 {
         class CF : public Base {
             class Rep;
             class Connector;
-            class Session;
+            class WSession;
             class ZSession;
             class Worker;
+            typedef boost::shared_ptr<ZSession> ZSessionPtr;
+            typedef boost::shared_ptr<WSession> WSessionPtr;
         public:
             ~CF();
             CF();
@@ -64,19 +66,17 @@ namespace metaproxy_1 {
         private:
             boost::scoped_ptr<Rep> m_p;
         };
-        class CF::Session : CF_Logger {
+        class CF::WSession : CF_Logger {
             friend class CF;
             friend class Rep;
             CF_Engine *engine;
             int age;
             std::string session_name;
             bool in_use;
-            bool destroyed;
             int log_file;
             std::string log_modules;
             mp::wrbuf file_area;
-            Session(std::string session_name);
-            ~Session();
+            WSession(std::string session_name);
             void log(int level, const char *fmt, ...);
             CF_Metadata metadata;
             bool module_check(const char *actual);
@@ -84,6 +84,7 @@ namespace metaproxy_1 {
             boost::mutex log_mutex;
             mp::wrbuf log_wr;
         public:
+            ~WSession();
             void printf(const char *module, int level, const char *fmt, ...);
         };
 
@@ -97,7 +98,7 @@ namespace metaproxy_1 {
             char *subdatabase;
             WRBUF username;
             WRBUF password;
-            CF::Session *cf_session;
+            WSessionPtr cf_session;
             int no_searches;
             bool get_next();
             struct json_node *get_detailed_record(int n, int *errcode,
@@ -106,8 +107,9 @@ namespace metaproxy_1 {
             bool get_record_xml(int n, WRBUF wrb, int *errcode, int details);
 
             ZSession(const Z_IdAuthentication *auth);
-            ~ZSession();
             void close();
+        public:
+            ~ZSession();
         };
 
         class CF::Worker {
@@ -123,7 +125,11 @@ namespace metaproxy_1 {
             void process_connector(metaproxy_1::Package & package,
                                    const char *suffix_path);
 
-
+            void cmd_log(metaproxy_1::Package & package, WSession *cf_session,
+                         Z_HTTP_Request *req,
+                         int no_parms, char **names, char **values);
+            void cmd_screen_shot(Package &package, WSession *cf_session,
+                                 Z_HTTP_Request *req);
             Z_Records *fetch_z3950(
                 CF::ZSession *zsession, mp::odr &odr,
                 const Odr_oid *preferredRecordSyntax,
@@ -146,23 +152,23 @@ namespace metaproxy_1 {
             CF_Engine *create_connector(CF_Engine::Flags engine_flag,
                                         const char *proxy,
                                         const char *loglevel,
-                                        CF::Session *ses,
+                                        CF::WSession *ses,
                                         std::string &addinfo);
 
             bool load_connector(Package &package, Z_HTTP_Request *req,
-                                CF::Session *cf_session);
+                                CF::WSession *cf_session);
 
             void task_connector(Package &package, Z_HTTP_Request *req,
-                                CF::Session *cf_session,
+                                CF::WSession *cf_session,
                                 const char *args);
             void expire(void);
 
             CF::ZSession *get_zsession(Package &package);
             void release_zsession(Package &package);
 
-            std::map<int,Session *> m_connectors;
+            std::map<int,WSessionPtr> m_connectors;
 
-            std::map<mp::Session,ZSession *> m_clients;
+            std::map<mp::Session,ZSessionPtr> m_clients;
 
             std::string http_prefix;
 
@@ -204,7 +210,6 @@ yf::CF::ZSession::ZSession(const Z_IdAuthentication *auth)
     subdatabase = 0;
     username = 0;
     password = 0;
-    cf_session = 0;
     no_searches = 0;
 
     if (auth)
@@ -268,8 +273,7 @@ void yf::CF::ZSession::close()
         }
     }
 
-    delete cf_session;
-    cf_session = 0;
+    cf_session.reset();
     xfree(database);
     xfree(subdatabase);
     database = 0;
@@ -437,7 +441,7 @@ bool yf::CF::ZSession::get_record_xml(int n, WRBUF wrb, int *errcode,
 }
 
 
-bool yf::CF::Session::module_check(const char *actual)
+bool yf::CF::WSession::module_check(const char *actual)
 {
     const char *cp = log_modules.c_str();
     if (!*cp)
@@ -457,8 +461,8 @@ bool yf::CF::Session::module_check(const char *actual)
     return false;
 }
 
-void yf::CF::Session::printf(const char *module, int level,
-                             const char *fmt, ...)
+void yf::CF::WSession::printf(const char *module, int level,
+                              const char *fmt, ...)
 {
     va_list ap;
     char buf[4096];
@@ -483,7 +487,7 @@ void yf::CF::Session::printf(const char *module, int level,
     va_end(ap);
 }
 
-void yf::CF::Session::log(int level, const char *fmt, ...)
+void yf::CF::WSession::log(int level, const char *fmt, ...)
 {
     const char *module = "engine";
     va_list ap;
@@ -504,17 +508,16 @@ void yf::CF::Session::log(int level, const char *fmt, ...)
     va_end(ap);
 }
 
-yf::CF::Session::Session(std::string a_session_name)
+yf::CF::WSession::WSession(std::string a_session_name)
     : session_name(a_session_name)
 {
     this->engine = 0;
     this->log_file = -1;
     this->in_use = true;
-    this->destroyed = false;
     this->age = 0;
 }
 
-yf::CF::Session::~Session()
+yf::CF::WSession::~WSession()
 {
     delete engine;
     if (log_file != -1)
@@ -576,18 +579,17 @@ void yf::CF::Rep::expire(void)
         {
             boost::mutex::scoped_lock lock(m_mutex);
 
-            std::map<int,Session *>::iterator map_it;
+            std::map<int,WSessionPtr>::iterator map_it;
             map_it = m_connectors.begin();
             while (map_it != m_connectors.end())
             {
-                Session *ses = map_it->second;
+                WSessionPtr ses = map_it->second;
                 if (!ses->in_use)
                 {
                     ses->age++;
                     if (ses->age > max_age)
                     {
                         m_connectors.erase(map_it);
-                        delete ses;
                         map_it = m_connectors.begin();
                     }
                     else
@@ -603,7 +605,7 @@ void yf::CF::Rep::expire(void)
 CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
                                          const char *proxy,
                                          const char *loglevel,
-                                         CF::Session *ses,
+                                         CF::WSession *ses,
                                          std::string &addinfo)
 {
     if (cf_factory->get_cf_tmp_dir(ses->file_area))
@@ -645,7 +647,7 @@ CF_Engine *yf::CF::Rep::create_connector(CF_Engine::Flags engine_flag,
 }
 
 bool yf::CF::Rep::load_connector(Package &package, Z_HTTP_Request *req,
-                                 CF::Session *ses)
+                                 CF::WSession *ses)
 {
 
     if (req->content_len > 0)
@@ -738,33 +740,31 @@ void yf::CF::Rep::post_connector(Package &package,
     }
 
     int id = 0;
-    Session *ses = 0;
+    WSessionPtr ses;
     {
         boost::mutex::scoped_lock lock(m_mutex);
         id = m_connector_next_id++;
         char session_name_str[20];
 
         sprintf(session_name_str, "%d", id);
-        ses = new Session(session_name_str);
+        ses = WSessionPtr(new WSession(session_name_str));
         ses->log_modules = logmodules;
     }
 
     std::string addinfo;
     CF_Engine *cf = create_connector(engine_flag, proxy, loglevel,
-                                     ses, addinfo);
+                                     ses.get(), addinfo);
     if (!cf)
     {
         Z_GDU *zgdu_res
             = odr.create_HTTP_Response(package.session(), req, 200);
         cf_http_msg(zgdu_res, odr, addinfo.c_str(), 500);
         package.response() = zgdu_res;
-        delete ses;
         return ;
     }
 
-    if (!load_connector(package, req, ses))
+    if (!load_connector(package, req, ses.get()))
     {
-        delete ses;
         return;
     }
 
@@ -811,7 +811,7 @@ static bool is_xml_content(const char *header)
 
 /** \brief operate on existing connector */
 void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
-                                 CF::Session *cf_session,
+                                 CF::WSession *cf_session,
                                  const char *args)
 {
     mp::odr odr;
@@ -961,14 +961,13 @@ void yf::CF::Rep::task_connector(Package &package, Z_HTTP_Request *req,
 void yf::CF::Rep::release_zsession(Package &package)
 {
     boost::mutex::scoped_lock lock(m_mutex);
-    std::map<mp::Session,ZSession *>::iterator it;
+    std::map<mp::Session,ZSessionPtr>::iterator it;
 
     it = m_clients.find(package.session());
     if (it != m_clients.end())
     {
         if (package.session().is_closed())
         {
-            delete it->second;
             m_clients.erase(it);
         }
         else
@@ -983,7 +982,7 @@ yf::CF::ZSession *yf::CF::Rep::get_zsession(Package &package)
 {
     boost::mutex::scoped_lock lock(m_mutex);
 
-    std::map<mp::Session,yf::CF::ZSession *>::iterator it;
+    std::map<mp::Session,yf::CF::ZSessionPtr>::iterator it;
 
     while (true)
     {
@@ -994,7 +993,7 @@ yf::CF::ZSession *yf::CF::Rep::get_zsession(Package &package)
         if (!it->second->in_use)
         {
             it->second->in_use = true;
-            return it->second;
+            return it->second.get();
         }
         m_cond_session_ready.wait(lock);
     }
@@ -1015,18 +1014,7 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
         !strcmp(sr->databaseNames[0], zsession->database)
         && zsession->no_searches < 20)
     {
-        try
-        {
-            const char *outp = 0;
-            zsession->cf_session->engine->run_task_opt("exit", "{}", &outp);
-        }
-        catch (CF_Engine_Error &e)
-        {
-            yaz_log(YLOG_WARN, "exit task returned error: %s", e.what());
-            zsession->close(); // zsession->database becomes 0
-        }
-        if (zsession->database)
-            return 0; // return if exit went OK
+        return 0; // same database as before: no need to open cf
     }
 
     zsession->close();
@@ -1087,7 +1075,7 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
         return a;
     }
 
-    Session *ses = 0;
+    WSessionPtr ses;
     {
         int id = 0;
         boost::mutex::scoped_lock lock(m_mutex);
@@ -1095,15 +1083,14 @@ Z_APDU *yf::CF::Rep::open_cf_z3950(Z_APDU *req,  CF::ZSession *zsession,
         char session_name_str[40];
 
         sprintf(session_name_str, "%lu.%d", session_id, id);
-        ses = new Session(session_name_str);
+        ses = WSessionPtr(new WSession(session_name_str));
     }
 
     std::string addinfo;
     CF_Engine *cf = create_connector(CF_Engine::FORK,
-                                     proxy, loglevel, ses, addinfo);
+                                     proxy, loglevel, ses.get(), addinfo);
     if (!cf)
     {
-        delete ses;
         return odr.create_searchResponse(req,
                                          YAZ_BIB1_TEMPORARY_SYSTEM_ERROR,
                                          addinfo.c_str());
@@ -1534,7 +1521,8 @@ void yf::CF::Rep::process_z3950(Package &package, CF::ZSession *zsession)
             else
             {
                 boost::mutex::scoped_lock lock(m_mutex);
-                m_clients[package.session()] = zsession;
+                m_clients[package.session()] =
+                    boost::shared_ptr<ZSession>(zsession);
             }
         }
         if (errcode)
@@ -1571,6 +1559,104 @@ void yf::CF::Rep::process_z3950(Package &package, CF::ZSession *zsession)
         package.response() = apdu_res;
 }
 
+void yf::CF::Rep::cmd_log(Package &package, WSession *cf_session,
+                          Z_HTTP_Request *req,
+                          int no_parms, char **names, char **values)
+{
+    mp::odr odr;
+    bool clear_log = false;
+    if (no_parms == 1)
+    {
+        if (!strcmp(names[0], "clear"))
+        {
+            if (values[0] && values[0][0] == '1')
+                clear_log = true;
+        }
+        else
+        {
+            Z_GDU *zgdu_res
+                = odr.create_HTTP_Response(package.session(), req, 200);
+            cf_http_msg(zgdu_res, odr, "bad argument for log", 400);
+            package.response() = zgdu_res;
+            return ;
+        }
+    }
+    Z_GDU *zgdu_res
+        = odr.create_HTTP_Response(package.session(), req, 200);
+    Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
+
+    z_HTTP_header_add_content_type(odr, &response->headers,
+                                   "text/plain", 0);
+    if (cf_session->log_file != -1)
+    {
+        int fd = cf_session->log_file;
+        mp::wrbuf cf_log_fname;
+        wrbuf_printf(cf_log_fname,
+                     "%s.log", wrbuf_cstr(cf_session->file_area));
+        long sz = lseek(fd, 0, SEEK_END);
+        if (sz > 0)
+        {
+            char *buf = (char*) odr_malloc(odr, sz);
+            pread(fd, buf, sz, 0);
+            response->content_buf = buf;
+            response->content_len = sz;
+            if (clear_log)
+            {
+                ftruncate(fd, 0);
+                lseek(fd, 0L, SEEK_SET);
+            }
+        }
+    }
+    package.response() = zgdu_res;
+}
+
+void yf::CF::Rep::cmd_screen_shot(Package &package, WSession *cf_session,
+                                  Z_HTTP_Request *req)
+{
+    mp::odr odr;
+    Z_GDU *zgdu_res = 0;
+    WRBUF fname = wrbuf_alloc();
+
+    wrbuf_printf(fname, "%s.png", wrbuf_cstr(cf_session->file_area));
+    try {
+        cf_session->engine->screen_shot(wrbuf_cstr(fname));
+    }
+    catch (CF_Engine_Error &e) {
+        zgdu_res = odr.create_HTTP_Response(package.session(), req, 500);
+        package.response() = zgdu_res;
+        wrbuf_destroy(fname);
+        return ;
+    }
+    FILE *file = fopen(wrbuf_cstr(fname), "rb");
+    if (!file)
+    {
+        zgdu_res = odr.create_HTTP_Response(package.session(), req, 500);
+        package.response() = zgdu_res;
+    }
+    else
+    {
+        zgdu_res  = odr.create_HTTP_Response(package.session(), req, 200);
+        Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
+        z_HTTP_header_add_content_type(odr, &response->headers,
+                                                       "image/png", 0);
+        fseek(file, 0L, SEEK_END);
+        long sz = ftell(file);
+        rewind(file);
+
+        char *buf = (char*) odr_malloc(odr, sz);
+        fread(buf, 1, sz, file);
+        fseek(file, sz, SEEK_SET);
+
+        response->content_buf = buf;
+        response->content_len = sz;
+
+        fclose(file);
+
+        package.response() = zgdu_res;
+    }
+    wrbuf_destroy(fname);
+}
+
 void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
 {
     Z_GDU *zgdu = package.request().get();
@@ -1633,11 +1719,11 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
             *cmd = 0;
         if (!strcmp(req->method, "POST"))
         {
-            Session *cf_session = 0;
+            WSessionPtr cf_session;
             {
                 boost::mutex::scoped_lock lock(m_mutex);
 
-                std::map<int,Session *>::const_iterator map_it;
+                std::map<int,WSessionPtr>::const_iterator map_it;
 
                 map_it = m_connectors.find(id);
                 if (map_it == m_connectors.end())
@@ -1650,102 +1736,14 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
                     return ;
                 }
                 cf_session = map_it->second;
-
                 if (!strcmp(cmd, "log"))
                 {
-                    bool clear_log = false;
-                    if (no_parms == 1)
-                    {
-                        if (!strcmp(names[0], "clear"))
-                        {
-                            if (values[0] && values[0][0] == '1')
-                                clear_log = true;
-                        }
-                        else
-                        {
-                            Z_GDU *zgdu_res
-                                = odr.create_HTTP_Response(package.session(), req, 200);
-                            cf_http_msg(zgdu_res, odr, "bad argument for log", 400);
-                            package.response() = zgdu_res;
-                            return ;
-                        }
-                    }
-                    Z_GDU *zgdu_res
-                        = odr.create_HTTP_Response(package.session(), req, 200);
-                    Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
-
-                    z_HTTP_header_add_content_type(odr, &response->headers,
-                                                   "text/plain", 0);
-                    if (cf_session->log_file != -1)
-                    {
-                        int fd = cf_session->log_file;
-                        mp::wrbuf cf_log_fname;
-                        wrbuf_printf(cf_log_fname,
-                                     "%s.log", wrbuf_cstr(cf_session->file_area));
-
-                        long sz = lseek(fd, 0, SEEK_END);
-                        if (sz > 0)
-                        {
-                            char *buf = (char*) odr_malloc(odr, sz);
-                            pread(fd, buf, sz, 0);
-                            response->content_buf = buf;
-                            response->content_len = sz;
-                            if (clear_log)
-                            {
-                                ftruncate(fd, 0);
-                                lseek(fd, 0L, SEEK_SET);
-                            }
-                        }
-                    }
-                    package.response() = zgdu_res;
+                    cmd_log(package, cf_session.get(), req, no_parms, names, values);
                     return;
                 }
                 else if (!strcmp(cmd, "screen_shot"))
                 {
-                    Z_GDU *zgdu_res = 0;
-                    WRBUF fname = wrbuf_alloc();
-
-                    wrbuf_printf(fname, "%s.png", wrbuf_cstr(cf_session->file_area));
-                    try {
-                        cf_session->engine->screen_shot(wrbuf_cstr(fname));
-                    }
-                    catch (CF_Engine_Error &e) {
-                        zgdu_res = odr.create_HTTP_Response(package.session(), req, 500);
-                        package.response() = zgdu_res;
-                        wrbuf_destroy(fname);
-                        return ;
-                    }
-                    FILE *file = fopen(wrbuf_cstr(fname), "rb");
-                    if (!file)
-                    {
-                        zgdu_res = odr.create_HTTP_Response(package.session(), req, 500);
-                        package.response() = zgdu_res;
-                    }
-                    else
-                    {
-                        zgdu_res  = odr.create_HTTP_Response(package.session(), req, 200);
-                        Z_HTTP_Response *response = zgdu_res->u.HTTP_Response;
-
-                        z_HTTP_header_add_content_type(odr, &response->headers,
-                                                       "image/png", 0);
-
-                        fseek(file, 0L, SEEK_END);
-
-                        long sz = ftell(file);
-                        rewind(file);
-
-                        char *buf = (char*) odr_malloc(odr, sz);
-                        fread(buf, 1, sz, file);
-                        fseek(file, sz, SEEK_SET);
-
-                        response->content_buf = buf;
-                        response->content_len = sz;
-
-                        fclose(file);
-
-                        package.response() = zgdu_res;
-                    }
-                    wrbuf_destroy(fname);
+                    cmd_screen_shot(package, cf_session.get(), req);
                     return;
                 }
                 // only one user at a time!
@@ -1761,31 +1759,22 @@ void yf::CF::Rep::process_connector(Package &package, const char *suffix_path)
             }
             // args is one of engine methods: "run_task", "run_task_opt",
             // "run_tests", "load_cf", "dom_string"
-            task_connector(package, req, cf_session, cmd);
+            task_connector(package, req, cf_session.get(), cmd);
             {
                 boost::mutex::scoped_lock lock(m_mutex);
                 cf_session->in_use = false;
                 cf_session->age = 0;
-                if (cf_session->destroyed)
-                    delete cf_session;
             }
         }
         else if (!strcmp(req->method, "DELETE"))
         {
             {
-                std::map<int,Session *>::iterator map_it;
+                std::map<int,WSessionPtr>::iterator map_it;
 
                 boost::mutex::scoped_lock lock(m_mutex);
                 map_it = m_connectors.find(id);
                 if (map_it != m_connectors.end())
-                {
-                    Session *ses = map_it->second;
                     m_connectors.erase(map_it);
-                    if (ses->in_use)
-                        ses->destroyed = true;
-                    else
-                        delete ses;
-                }
             }
             Z_GDU *zgdu_res
                 = odr.create_HTTP_Response(package.session(), req, 200);
